@@ -14,8 +14,13 @@ import time
 import os
 import signal
 from distutils.sysconfig import get_config_var
+from distutils.version import LooseVersion
 import py2app
 import platform
+if __name__ == "__main__":
+    from tools import kill_child_processes
+else:
+    from .tools import kill_child_processes
 
 DIR_NAME=os.path.dirname(os.path.abspath(__file__))
 
@@ -30,6 +35,8 @@ class TestBasicPlugin (unittest.TestCase):
     # a base-class.
     @classmethod
     def setUpClass(cls):
+        kill_child_processes()
+
         try:
             if os.path.exists(os.path.join(cls.plugin_dir, 'build')):
                 shutil.rmtree(os.path.join(cls.plugin_dir, 'build'))
@@ -40,6 +47,7 @@ class TestBasicPlugin (unittest.TestCase):
             cmd = [ sys.executable, 'setup.py', 'py2app'] + cls.py2app_args
 
             env=os.environ.copy()
+            env['TMPDIR'] = os.getcwd()
             pp = os.path.dirname(os.path.dirname(py2app.__file__))
             if 'PYTHONPATH' in env:
                 env['PYTHONPATH'] = pp + ':' + env['PYTHONPATH']
@@ -55,11 +63,15 @@ class TestBasicPlugin (unittest.TestCase):
                 cwd = cls.plugin_dir,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.STDOUT,
-                close_fds=True,
+                close_fds=False,
                 env=env)
             lines = p.communicate()[0]
             if p.wait() != 0:
                 print (lines)
+                try:
+                    os.waitpid(0, 0)
+                except os.error:
+                    pass
                 raise AssertionError("Creating basic_plugin bundle failed")
 
             p = subprocess.Popen([
@@ -68,13 +80,17 @@ class TestBasicPlugin (unittest.TestCase):
             lines = p.communicate()[0]
             xit = p.wait()
             if p.wait() != 0:
+                try:
+                    os.waitpid(0, 0)
+                except os.error:
+                    pass
                 raise AssertionError("Fetching Xcode root failed")
 
             root = lines.strip()
             if sys.version_info[0] != 2:
                 root = root.decode('utf-8')
 
-            if platform.mac_ver()[0] < '10.7.':
+            if LooseVersion(platform.mac_ver()[0]) < LooseVersion('10.7'):
                 cc = [get_config_var('CC')]
                 env = dict(os.environ)
                 env['MACOSX_DEPLOYMENT_TARGET'] = get_config_var('MACOSX_DEPLOYMENT_TARGET')
@@ -89,12 +105,21 @@ class TestBasicPlugin (unittest.TestCase):
                 env=env,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.STDOUT,
-                close_fds=True)
+                close_fds=False)
             lines = p.communicate()[0]
             if p.wait() != 0:
                 print (lines)
+                try:
+                    os.waitpid(0, 0)
+                except os.error:
+                    pass
                 raise AssertionError("Creating bundle_loader failed")
 
+            try:
+                os.waitpid(0, 0)
+            except os.error:
+                pass
+
 
         except:
             cls.tearDownClass()
@@ -111,6 +136,8 @@ class TestBasicPlugin (unittest.TestCase):
         if os.path.exists(os.path.join(cls.plugin_dir, 'dist')):
             shutil.rmtree(os.path.join(cls.plugin_dir, 'dist'))
 
+        time.sleep(2)
+
     def start_app(self):
         # Start the test app, return a subprocess object where
         # stdin and stdout are connected to pipes.
@@ -118,14 +145,18 @@ class TestBasicPlugin (unittest.TestCase):
                     os.path.join(self.plugin_dir,
                                 'dist/BasicPlugin.bundle'),
         ]
-        p = subprocess.Popen(cmd,
+        p = self._p = subprocess.Popen(cmd,
                 stdin=subprocess.PIPE,
                 stdout=subprocess.PIPE,
-                close_fds=True,
+                close_fds=False,
                 )
                 #stderr=subprocess.STDOUT)
         return p
 
+    def tearDown(self):
+        kill_child_processes()
+        time.sleep(1)
+
     def wait_with_timeout(self, proc, timeout=10):
         for i in range(timeout):
             x = proc.poll()
@@ -159,6 +190,17 @@ class TestBasicPlugin (unittest.TestCase):
         exit = self.wait_with_timeout(p)
         self.assertEqual(exit, 0)
 
+    def test_python_executable_mode(self):
+        path = os.path.join(
+            self.plugin_dir,
+            'dist/BasicPlugin.bundle/Contents/MacOS/python')
+
+        self.assertTrue(os.path.exists(path))
+        mode = os.stat(path).st_mode
+        self.assertTrue(mode & 0o001, 'Not executable for other')
+        self.assertTrue(mode & 0o010, 'Not executable for group')
+        self.assertTrue(mode & 0o100, 'Not executable for user')
+
 class TestBasicAliasPlugin (TestBasicPlugin):
     py2app_args = [ '--alias' ]
 
